import 'package:adeptarch/adeptarch.dart';
import 'package:adeptutils/adeptutils.dart';
import 'package:graphql_flutter/graphql_flutter.dart';

class QueryResponse<T> {

  DateTime timestamp;

  /// The source of the result data.
  ///
  /// null when unexecuted.
  /// Will be set when encountering an error during any execution attempt
  QueryResultSource source;

  T data;

  OperationException exception;

  QueryResponse({
    this.data,
    this.exception,
    bool loading,
    bool optimistic,
    QueryResultSource source,
    DateTime timestamp,
  }) : timestamp = timestamp ?? DateTime.now(),
        this.source = source ??
            ((loading == true)
                ? QueryResultSource.Loading
                : (optimistic == true)
                ? QueryResultSource.OptimisticResult
                : null);

  /// Whether data has been specified from either the cache or network)
  bool get loading => source == QueryResultSource.Loading;

  /// Whether an optimistic result has been specified
  ///   (may include eager results from the cache)
  bool get optimistic => source == QueryResultSource.OptimisticResult;

  /// Whether the response includes an exception
  bool get hasException => exception != null;

  /// Whether the response includes an exception
  bool get isSuccessful => exception == null;

//  static QueryResponse fromQueryResult(
//      QueryResult queryResult, DataTypeResponseConverter dataTypeResponseConverter){
//    var data;
//    try{
//      data = dataTypeResponseConverter(queryResult);
//    }catch(e){
//      print(e);
//    }
//    return QueryResponse(
//      data: data,
//      exception: queryResult?.exception,
//      source: queryResult?.source,
//      timestamp: queryResult.timestamp,
//    );
//  }
//
//  static QueryResponse fromQueryResultOld<DataType extends Jsonable>(
//      QueryResult queryResult, DataType dataType,
//      DataTypeResponseConverter dataTypeResponseConverter){
//    DataType data;
//    try{
//      if(queryResult?.data != null)
//        return queryResult?.data is Map
//            ? fromMapQueryResult(queryResult, dataType, dataTypeResponseConverter)
//            : fromListQueryResult(queryResult, dataType, dataTypeResponseConverter);
//    }catch(e){
//      print(e);
//    }
//    return QueryResponse<DataType>(
//      data: data,
//      exception: queryResult?.exception,
//      source: queryResult?.source,
//      timestamp: queryResult.timestamp,
//    );
//  }

  static QueryResponse<DataType> fromMapQueryResult<DataType>(
      QueryResult queryResult,
      DataTypeResponseConverter<DataType> dataTypeResponseConverter,){
    DataType data;
    try{
      data = dataTypeResponseConverter(queryResult);
    }catch(e){
      print(e);
    }
    return QueryResponse<DataType>(
      data: data,
      exception: queryResult?.exception,
      source: queryResult?.source,
      timestamp: queryResult.timestamp,
    );
  }

  static QueryResponse<List<DataType>> fromListQueryResult<DataType>(
      QueryResult queryResult,
      DataTypeResponseConverter<List<DataType>> dataTypeResponseConverter){
    List<DataType> data;
    try{
      data = dataTypeResponseConverter(queryResult);
    }catch(e){
      print(e);
    }
    return QueryResponse<List<DataType>>(
      data: data,
      exception: queryResult?.exception,
      source: queryResult?.source,
      timestamp: queryResult.timestamp,
    );
  }
}