import 'package:adeptutils/adeptutils.dart';
import 'package:graphql_flutter/graphql_flutter.dart';

typedef DataTypeResponseConverter<T> = T Function(QueryResult queryResult);

class QueryResponseConverter<T> {
  T data;
  DataTypeResponseConverter<List<T>> toDataListResponseConverter;
  DataTypeResponseConverter<T> toDataResponseConverter;
  QueryResponseConverter({
    this.data,
  }) {
    toDataListResponseConverter = (queryResult) => toDataList(queryResult);
    toDataResponseConverter = (queryResult) => toData(queryResult);
  }

  List<T> toDataList(QueryResult queryResult) {
    try{
      if(data == null || queryResult?.data == null) return null;
      List list;
      if(queryResult.data is List)
        list = queryResult.data;
      else
      if(queryResult.data is Map && (queryResult.data as Map).isNotEmpty)
        list = (queryResult.data as Map).values.first;
      if(data is Jsonable)
        return (data as Jsonable).fromJsonList(list);
      if(list.runtimeType == data.runtimeType)
        return list;
    }catch(e){
      print(e);
    }
    return null;
  }

  T toData(QueryResult queryResult) {
    try{
      if(data == null || queryResult?.data == null) return null;
      if(data is Jsonable && queryResult.data is Map && (queryResult.data as Map).isNotEmpty)
        return (data as Jsonable).fromJsonMap((queryResult.data as Map).values.first);
      if(queryResult.data?.runtimeType == data.runtimeType)
        return queryResult.data;
    }catch(e){
      print(e);
    }
    return null;
  }

}